#!/usr/bin/perl -w

use strict;
use Getopt::Long;

my %opts = ('deprecation'=>1); # default to deprecation on

GetOptions(\%opts, "help!", "quiet!", "repeat!", "deprecation!");

if ($opts{'help'}) {
    &help;
    exit 0;
}

if (grep /-help/, @ARGV) {
    exec ("jikes") || die ("Unable to run jikes: $!\n");
}

my @files;
my ($cmdLine, $dir, $product);

($dir) = $0 =~ /^(.*?)\/[^\/]*$/;
chdir $dir;
chomp ($dir = qx|pwd|);
($product) = $dir =~ /([^\/]*)$/;

open FILES, "find ./src -name '*.java' -print|";
chomp (@files = <FILES>);
close FILES;

my $classpath = $ENV{'JIKESPATH'} || $ENV{'CLASSPATH'} || "";
    
$cmdLine = 'jikes -Xstdout ' 
  . ($opts{'deprecation'}?'-deprecation ':'')
  . (join ' ', @ARGV) 
  . " -d ./classes -classpath ./classes:$classpath "
  . (join ' ', @files);

@files = (); # free mem

my ($errors, $result, $firstTime) = (0, 0,1);

while ($firstTime || $opts{'repeat'}) {
  unless ($firstTime) {
    print "*** Running again. $errors errors on last attempt.\n" 
  }
  $firstTime = 0;
  $_ = &compile;
  if (!$_ || $errors == $_) {  # done
    $errors = $_; # incase 0
    last;
  }
  $errors = $_;
}
if ($errors) {
  print "*** Failed compilation with $errors errors.\n";
} else {
  print "*** Succesful $product build.\n";
}
if (-e "./core") {
    print "*** ./core exists -- something ugly may have happened.\n";
}
exit $errors;

######################[ subroutines ]############################

sub compile { 
  my $count = 0;
  open OUTPUT, "$cmdLine |";
  foreach (<OUTPUT>) {
    print unless $opts{'quiet'};
    if (/^Found (\d+) /) {
      $count += $1;
    }
  }
  close OUTPUT;
  return $count;
}

sub help { 
    exec "pod2man $0 | groff -mandoc -Tascii";
    die ("Unable to find pod2man or groff\n");
}

__END__

########################[ pod ]##################################

=head1 NAME

jikesbuild - perl script to compile java source branches

=head1 USAGE

jikesbuild [options] -- [options_for_jikes]

=head1 OPTIONS

All options can be abbreviated to the shortest non-ambiguous string possible

=over 4

=item -repeat

Causes jikes to be run repeatedly until there the number of lines
of output are the same for two consecutive runs.

=item -quiet

Supresses compiler output.

=item -help

Prints help for jikesbuild (note: use '-- -help' to get help on jikes)

=item -nodeprecation

Don't show warnings about deprecated items

=item options_to_jikes

Additional parameters to pass to each run of jikes

=head1 DESCRIPTION

When jikes is called a -classpath option will be passed to it, thus preventing
jikes from looking at the $CLASSPATH or $JIKESPATH variables.  The
-classpath it is given is constructed using the following algorithm:

    if (defined $JIKESPATH environment variable) {
	use -classpath of ./classes:$JIKESPATH 
    } else {
	use -classpath of ./classes:$CLASSPATH
    }


All of the source files in the ./src directory and its children will be
included in the compile. 

=back

=head1 AUTHOR

Ry4an Brase <ry4an@ry4an.org>

=cut
